#+BEGIN_COMMENT
.. title: Unittest objects available on DBus: part 2
.. slug: unittest-objects-available-on-dbus-part-2
.. date: 2019-03-13 09:43:38 UTC+05:30
.. tags: mock, python, unittest, dbus
.. category: coding
.. link: 
.. description: Better way to unittest Objects/Interfaces meant to be exported over DBus
.. type: text
#+END_COMMENT


This is a follow up of my [[https://baali.muse-amuse.in/posts/unittest-objectinterface-available-on-dbus][previous post]]. Basically I am learning
things about [[https://docs.python.org/3/library/unittest.mock-examples.html][~mock~]] library and [[https://pypi.org/project/dbus-python/][DBus]] as [[https://www.senic.com/en/about][we]] are removing technical
debt, addressing our pain points and making product stable.


* Partial mock DBus to unittest Object exported over DBus

[[https://baali.muse-amuse.in/posts/unittest-objectinterface-available-on-dbus][last time]] I wrote about the pain to test ~object~ available on DBus. I
got it reviewed with my teammates. As I paired with one, he agreed
that with such a setup, these tests are not ~unittest~ but more like
system or integration test.

First thing, let me share how I am exporting ~Object~ on ~dbus~:
#+BEGIN_SRC python
  #!/usr/bin/env python3

  import dbus
  import dbus.service
  import json

  INTERFACE_NAME = "org.example.ControlInterface"

  class ControlObject(dbus.service.Object):
      def __init__(self, bus_name, conf):
          super().__init__(bus_name, "/org/example/ControlObject")
          self._conf = conf
          print("Started service")

      @dbus.service.method(INTERFACE_NAME,
                           in_signature='s', out_signature='b')
      def Update(self, conf):
          try:
              self._conf = json.loads(conf)
          except json.JSONDecodeError:
              print('Could not parse json')
              raise

      @dbus.service.method(INTERFACE_NAME,
                           in_signature='s', 
                           out_signature='s')
      def Get(self, key=''):
          if key == '':
              # we can have empty strings as key to dict in python
              raise KeyError
          try:
              components = _conf[key]
          except (KeyError, TypeError):
              raise
          else:
              try:
                  component = next(c for c in components if c['id'] == component_id)
              except StopIteration:
                  raise
              else:
                  return json.dump(component)
#+END_SRC

This ~Object~ takes a [[https://dbus.freedesktop.org/doc/dbus-tutorial.html#bus-names][~bus_name~]] argument for initializing:

#+BEGIN_SRC python
  try:
      bus_name = dbus.service.BusName("org.example.ControlInterface",
                                      bus=dbus.SystemBus(),
                                      do_not_queue=True)
  except dbus.exceptions.NameExistsException:
      logger.info("BusName is already used by some different service")
  else:
      ControlObject(bus_name, {})
#+END_SRC

This way of setting up things coupled my ~Object~ to DBus setup
tightly. I have to pass this ~bus_name~ as argument. As I was getting
it reviewed with another of my colleague, he mentioned that I should
be able to ~patch~ ~dbus~ and possibly get around with [[https://baali.muse-amuse.in/posts/unittest-objectinterface-available-on-dbus][way I was setting up system with DBus, export object to it and then run test]].

I had used [[https://docs.python.org/3/library/unittest.mock-examples.html#partial-mocking][partial mocking]] using ~with~ construct of ~mock~, I put
together following test using it:

#+BEGIN_SRC python
  from unittest import mock
  from unittest import TestCase
  import json
  from service import ControlObject

  class TestService(TestCase):
      def setUp(self):
          with mock.patch('service.dbus.SystemBus') as MockBus:
              self.obj = ControlObject(MockBus, {})

      def tearDown(self):
          del self.obj

      def test_object_blank_state(self):
          self.assertFalse(self.obj._conf)

      def test_object_update_random_string(self):
          exception_string = 'Expecting value'
          with self.assertRaises(json.JSONDecodeError) as context:
              self.assertFalse(self.obj.Update(''))
          self.assertIn(exception_string, context.exception.msg)
          self.assertFalse(self.obj._conf)
        
          with self.assertRaises(json.JSONDecodeError) as context:
              self.assertFalse(self.obj.Update('random string'))
          self.assertIn(exception_string, context.exception.msg)
          self.assertFalse(self.obj._conf)

      def test_object_update(self):
          conf = {'id': 'id',
                  'name': 'name',
                  'station': 'station'}
          self.obj.Update(json.dumps(conf))

          self.assertTrue(self.obj._conf)
          for key in conf:
              self.assertTrue(key in self.obj._conf)
              self.assertEqual(conf[key], self.obj._conf[key])
#+END_SRC

This test worked directly and there was no need to, setup
~dbus-session~ on docker image, run a process where I export the
object and call methods over DBus. Instead now, I can directly access
all attributes and methods of the ~Object~ and write better tests.

#+BEGIN_SRC shell
$ python -m unittest -v test_service.py
test_object_blank_state (test_service.TestService) ... ok
test_object_update (test_service.TestService) ... ok
test_object_update_random_string (test_service.TestService) ... Could not parse json
Could not parse json
ok

----------------------------------------------------------------------
Ran 3 tests in 0.002s

OK
#+END_SRC

* Another way(Better?) to export Object over DBus

While writing this post and exploring examples from [[https://pypi.org/project/dbus-python/][dbus-python]] I
found another way to write class which can be exported over DBus:

#+BEGIN_SRC python
  class ControlObject(dbus.service.Object):
      def __init__(self, conf, *args, **kwargs):
          super().__init__(*args, **kwargs)
          self._conf = conf
          print("Started service")
#+END_SRC

Now we don't even need to claim ~BusName~ and pass it as argument to
this class. Instead we can make this ~Object~ available on DBus by:

#+BEGIN_SRC python
  system_bus = dbus.SystemBus()
  bus_name = dbus.service.BusName('org.example.ControlObject',
                                  system_bus)
  ComponentsService({}, system_bus, '/org/example/ControlObject')
#+END_SRC

And when we don't want to use DBus and just create instance of this
~Object~ we can directly do that also, by calling
~ComponentsService({})~ directly. With this way of initializing, we
don't need to partial ~mock~ DBus and write ~unittest~ directly.
